###########################################################################
# Export of Script Module: netmri_easy
# Language: Python
# Category: Internal
# Description: Object oriented library for Python scripting support
###########################################################################
import datetime
import sys
import time
from urllib.parse import urlparse

from infoblox_netmri.client import InfobloxNetMRI


class NetMRIEasy(object):
    def __init__(self, debug=False, **kwargs):
        self.debug = debug
        self.api_version = kwargs.get('api_version') or "auto"
        self.host = urlparse(kwargs.get('api_url')).hostname
        self.username = kwargs.get('http_username')
        self.password = kwargs.get('http_password')
        self.job_id = kwargs.get('job_id')
        self.device_id = kwargs.get('device_id')
        self.batch_id = kwargs.get('batch_id')
        self.script_login = kwargs.get('script_login')

        if (not self.job_id) or (not self.device_id) or (not self.batch_id):
            raise RuntimeError('job_id or device_id or batch_id not initialized')

        self.client = InfobloxNetMRI(
            self.host,
            self.username,
            self.password,
            api_version=self.api_version
        )

        self.dis_session = self._open_dis_session()
        if not self.script_login == 'false':
            self.cli_connection = self._open_cli_connection()
        self._setup_logging()

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.close_session()

    def broker(self, name):
        return self.client.get_broker(name)

    def _setup_logging(self):
        broker = self.broker('Job')
        if hasattr(broker, 'log_custom_message'):
            def remote_log_message(self, severity, message):
                date = datetime.datetime.now()
                brkr = self.broker('Job')
                msg_formatted = "{} [{:<7}] {}".format(
                    date.strftime("%Y-%m-%d %H:%M:%S "),
                    str(severity).upper(),
                    message
                )
                brkr.log_custom_message(message=msg_formatted, id=self.batch_id,
                                        jobdetailid=self.job_id, severity=severity)

            setattr(self.__class__, "remote_log_message", remote_log_message)
        else:
            def remote_log_message(self, severity, message):
                if not hasattr(self, 'cli_connection'):
                    return

                brkr = self._get_cli_broker()
                return brkr.log_message(
                    id=self.dis_session.SessionID,
                    device_id=self.device_id,
                    severity=severity,
                    message=message
                )

            setattr(self.__class__, "remote_log_message", remote_log_message)

    def _open_cli_connection(self):
        if getattr(self, 'cli_connection', None) and self.cli_connection:
            return self.cli_connection
        return self._get_cli_broker().open(
            id=self.dis_session.SessionID,
            DeviceID=self.device_id
        )

    def _close_dis_session(self):
        return self.broker('DisSession').close(id=self.dis_session.SessionID)

    def _open_dis_session(self):
        return self.broker('DisSession').open(job_id=self.job_id)

    def send_command(self, command, regex=None):
        if not hasattr(self, 'cli_connection'):
            return

        regex = regex or ""
        result = self._get_cli_broker().send_command(
            id=self.dis_session.SessionID,
            device_id=self.device_id,
            command=command,
            regex=regex
        )
        if result:
            return result.get('command_response')
        return result

    def send_async_command(self, command, timeout, regex, wait_until_finished=True):
        if not hasattr(self, 'cli_connection'):
            return

        regex = regex or ""
        self._print("Sending asynchronous command ({})".format(command))
        try:
            async_command_result = self._get_cli_broker().send_async_command(
                id=self.dis_session.SessionID,
                device_id=self.device_id,
                command=command,
                timeout=timeout,
                debug=self.debug,
                regex=regex,
            )
            self._ok()
            async_command_id = async_command_result.get('command_response')

            if async_command_id:
                async_command_id = async_command_id.replace("async_command_id:", "")
            else:
                self._error("Invalid send_async_command response ({})".format(str(async_command_id)))

            if not async_command_id.isdigit():
                self._error("Invalid send_async_command response ({})".format(str(async_command_id)))

            self._print("Received async_command_id {}".format(str(async_command_id)))
            self._ok()
            if not wait_until_finished:
                return async_command_id

            delay = 30
            max_delay = 14400 + 900  # DIS max session time plus a little padding (4.25 hours)
            cur_delay = 0
            ok = "async_command_id_status:OK\n"
            error = "async_command_id_status:Error\n"
            while True:
                self._print("Getting the status of async_command_id {}".format(async_command_id))
                res = self._get_cli_broker().get_async_command_status(
                    id=self.dis_session.SessionID,
                    device_id=self.device_id,
                    async_command_id=async_command_id
                )

                if not res:
                    self._error("Invalid get_async_command_status response ({})".format(res))
                result = res.get('command_response')

                if ok in result:
                    return result.replace(ok, '')

                if error in result:
                    self._error("Asynchronous command failed {}".format(
                        result.replace(error, '')
                    ))

                if cur_delay >= max_delay:
                    self._error("Timeout waiting for asynchronous command to complete")
                time.sleep(delay)
                cur_delay += delay

        except AttributeError:
            raise RuntimeError("Api version {} not support async commands. Minimal version: 2.10".format(
                self.client.api_version
            ))

    def get_config(self, sync=True):
        if not hasattr(self, 'cli_connection'):
            return

        self._print("Requesting on demand configuration collection ({})".format(sync))
        traking_id = None
        try:
            res = self.broker('ConfigRevision').get_configs(
                DeviceID=self.device_id
            )
            traking_id = res.get('TrackingID')

        except RuntimeError:
            raise RuntimeError("Api version {} not support this command. Minimal version: 2.10".format(
                self.client.api_version
            ))
        self._print("Received TrackingID {}".format(traking_id))
        self._ok()
        if not sync:
            return traking_id

        delay = 30
        max_delay = 600
        cur_delay = 0
        while True:
            self._print("Getting the status of TrackingID {}".format(traking_id))
            status_resp = self.broker('ConfigRevision').get_configs_status(
                TrackingID=traking_id
            )
            if status_resp.get('Status') == "OK":
                return status_resp

            if status_resp.get('Status') == "Error":
                self._error("On demand configuration collection failed")

            if cur_delay >= max_delay:
                self._error("Timeout waiting for configuration collection to complete")

            time.sleep(delay)
            cur_delay += delay

            self._print("Sending 'Keep Alive CR/LF'")
            self.send_command("NOP:")
            self._ok()

    def set_variable(self, name, value):
        if not hasattr(self, 'cli_connection'):
            return

        command = '${name} ="{value}"'.format(name=name, value=value)
        result = self._get_cli_broker().set_variable(
            id=self.dis_session.SessionID,
            device_id=self.device_id,
            command=command
        )
        if result:
            return result.get('command_response')
        return result

    def get_template(self, template_name, stage):
        if not hasattr(self, 'cli_connection'):
            return

        result = self._get_cli_broker().get_template(
            id=self.dis_session.SessionID,
            device_id=self.device_id,
            template=template_name,
            stage=(stage or 0)
        )
        if result:
            return result.get('command_response')
        return result

    def get_list_value(self, list_name, key_column, key_value, value_column, default):
        if not hasattr(self, 'cli_connection'):
            return

        result = self._get_cli_broker().get_list_value(
            id=self.dis_session.SessionID,
            device_id=self.device_id,
            list_name=list_name,
            key_column=key_column,
            key_value=key_value,
            value_column=value_column,
            default_value=default
        )
        if result:
            return result.get('command_response')
        return result

    def generate_issue(self, issue_type_id, severity, **kwargs):
        result = self.broker('IssueAdhoc').generate_issue(
            DeviceID=self.device_id,
            BatchID=self.batch_id,
            Severity=severity,
            IssueTypeID=issue_type_id,
            **kwargs
        )
        if result:
            return result.get('IssueID')
        return result

    def close_session(self):
        brkr = self.broker('DisSession')
        return brkr.close(
            SessionID=self.dis_session.SessionID
        )

    def get_device(self):
        return self.broker('Device').show(
            DeviceID=self.device_id
        )

    def log_message(self, severity, message):
        self.remote_log_message(severity, message)

    def _get_cli_broker(self):
        return self.client.get_broker('CliConnection')

    def _print(self, msg):
        if self.debug:
            print(msg)

    def _print_status(self, status):
        if self.debug:
            print(status)

    def _ok(self):
        self._print("OK")

    def _error(self, message):
        print("\n*** ERROR: {} ***\n".format(message))
        sys.exit(-1)



from ..broker import Broker


class BasicServicesBroker(Broker):
    controller = "basic_services"

    def authenticate(self, **kwargs):
        """Authenticates the user with NetMRI.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param username: The username of the user as whom to login.
             :type username: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param password: The password of the user as whom to login.
             :type password: String

            |  ``api version min:`` 2.3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` %Y-%m-%d %H:%M:%S

             :param datetime_format: The format to use for date/time input and output.
             :type datetime_format: String

            |  ``api version min:`` 2.3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param timezone: Date/time input and output will be performed in the specified timezone. Should be specified as HH:MM offset from GMT. For example, -05:00 specified US Eastern Time, whereas +09:00 specifies Tokyo time. Alternatively, a timezone name may be used. See the API Data Structures page for details. If omitted, the server's configured timezone will be used.
             :type timezone: String

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("authenticate"), kwargs)

    def base_uri(self, **kwargs):
        """Returns the base URI for the specified version.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param version: The API version for which the base_uri is needed.
             :type version: String

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("base_uri"), kwargs)

    def license_info(self, **kwargs):
        """Returns license information for this NetMRI server.

            **Inputs**

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return serial_number: NetMRI serial number.
             :rtype serial_number: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return license_id: NetMRI License identifier.
             :rtype license_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return license_expiration: NetMRI License expiration.
             :rtype license_expiration: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return license_type: NetMRI License type
             :rtype license_type: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return mode: NetMRI operation mode. One of 'standalone', 'master' or 'collector'.
             :rtype mode: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return maintenance_expiration: Maintenance expiration for appliance.
             :rtype maintenance_expiration: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return device_limit: Licensed limit of devices.
             :rtype device_limit: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return interface_limit: Licensed limit of interfaces.
             :rtype interface_limit: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return spm_limit: Licensed limit of number of ports controlled by SPM.
             :rtype spm_limit: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return modules_short_name: Short symbolic names of licensed features.
             :rtype modules_short_name: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return modules_support: Support statuses for corresponding modules in modules_short_names.
             :rtype modules_support: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return modules_expiration: Expiration times for corresponding modules in modules_short_names.
             :rtype modules_expiration: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return modules_name: Long names for corresponding modules in modules_short_names.
             :rtype modules_name: Array of String

            """

        return self.api_request(self._get_method_fullname("license_info"), kwargs)

    def server_info(self, **kwargs):
        """Returns basic information regarding this NetMRI server.

            **Inputs**

            |  ``api version min:`` 2.6
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` False

             :param api_versions_only_ind: Only include API version information in the output.
             :type api_versions_only_ind: Boolean

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return netmri_version: The NetMRI version number running on this appliance or virtual machine.
             :rtype netmri_version: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return latest_api_version: The most recent API version supported by this NetMRI.
             :rtype latest_api_version: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return requested_api_version: The API version that executed this call.
             :rtype requested_api_version: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return host_name: The configured host name of the NetMRI appliance.
             :rtype host_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return operating_mode: Indicates if the NetMRI is running in standalone, collector, or operations center mode.
             :rtype operating_mode: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return mgmt_ip: The IPv4 management address of this NetMRI, if configured.
             :rtype mgmt_ip: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return mgmt_ip6: The IPv6 management address of this NetMRI, if configured.
             :rtype mgmt_ip6: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return scan_ip: The IPv4 SCAN (analysis) address of this NetMRI, if configured.
             :rtype scan_ip: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return scan_ip6: The IPv6 SCAN (analysis) address of this NetMRI, if configured.
             :rtype scan_ip6: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return operational_status: The status of NetMRI. Usually ready, can also be upgrading. Values might change in the future.
             :rtype operational_status: String

            |  ``api version min:`` 2.3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return supported_api_versions: All API versions supported by this NetMRI.
             :rtype supported_api_versions: Array of String

            """

        return self.api_request(self._get_method_fullname("server_info"), kwargs)

    def server_time(self, **kwargs):
        """Returns the current system time of this NetMRI server.

            **Inputs**

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("server_time"), kwargs)

    def restart(self, **kwargs):
        """Restarts the application.

            **Inputs**

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("restart"), kwargs)

    def consolidate(self, **kwargs):
        """Runs consolidation

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param managers: Comma-separated list of consolidator managers. Must be one of Aggregate, Config, Event, Group, Issue, Job, Normal, Policy, Routing, Settings, Stats, Subnet, Switching, Time, Topology, Voip, Vulnerability, Wireless
             :type managers: Array

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param collector: Collector name. In case when this method called on OC this parameter is required
             :type collector: String

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("consolidate"), kwargs)

    def settings_generate(self, **kwargs):
        """Generates xml with current configuration data

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param version: The version of xml to be generated
             :type version: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return xml: A string containing the full xml as collected from the running config.
             :rtype xml: String

            """

        return self.api_request(self._get_method_fullname("settings_generate"), kwargs)

    def settings_current(self, **kwargs):
        """Reports the status of an xml configuration file

            **Inputs**

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return xml: A string containing the full xml as collected from the running config.
             :rtype xml: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return messages: An array of hashes that contain details about the validation process
             :rtype messages: Array of Hash

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return status: A string representation of the status of the request. Will be one of; success, error, pending
             :rtype status: String

            """

        return self.api_request(self._get_method_fullname("settings_current"), kwargs)

    def settings_apply(self, **kwargs):
        """Parses the xml provided by config_id, then applies the changes. You should not need to call this directly!

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param config_id: The configuration id reported when the xml was uploaded to the unit
             :type config_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param mods: Modifications for applying
             :type mods: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return xml: A string containing the full xml as collected from the running config.
             :rtype xml: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return messages: An array of hashes that contain details about the validation process
             :rtype messages: Array of Hash

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return status: A string representation of the status of the request. Will be one of; success, error, pending
             :rtype status: String

            """

        return self.api_request(self._get_method_fullname("settings_apply"), kwargs)

    def settings_status(self, **kwargs):
        """Reports the status of an xml configuration file

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param config_id: The configuration id reported when the xml was uploaded to the unit
             :type config_id: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return messages: An array of hashes that contain details about the validation process
             :rtype messages: Array of Hash

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return status: A string representation of the status of the validation. Will be one of; success, error, pending
             :rtype status: String

            """

        return self.api_request(self._get_method_fullname("settings_status"), kwargs)

    def settings_info(self, **kwargs):
        """Shows probe info, running_config, candidate_config, and list of installed dsb

            **Inputs**

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return grid_members: Hash of grid members info including master and slaves (probes)
             :rtype grid_members: String

            """

        return self.api_request(self._get_method_fullname("settings_info"), kwargs)

    def set_session_value(self, **kwargs):
        """save data in a cache that is session wise

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param key: key associated with that value - will be used to retrieve the same value
             :type key: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param value: value to save in the session cache
             :type value: String

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("set_session_value"), kwargs)

    def get_session_value(self, **kwargs):
        """retrieve data in the session cache that formerly saved

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param key: key associated with that value - will be used to retrieve the same value
             :type key: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param default_value: Default value in case key/value does not exist in session. If key does not exist and default value is nil the response is 400 with record not found message
             :type default_value: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return value: value associated with that key
             :rtype value: String

            """

        return self.api_request(self._get_method_fullname("get_session_value"), kwargs)

    def set_time_zone(self, **kwargs):
        """Sets a Time Zone for the system.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param time_zone: A valid Time Zone in "Area/Location" format.
             :type time_zone: String

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("set_time_zone"), kwargs)

    def get_current_user(self, **kwargs):
        """Returns current logged user information

            **Inputs**

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return user_info: None
             :rtype user_info: Hash

            """

        return self.api_request(self._get_method_fullname("get_current_user"), kwargs)



from ..broker import Broker


class DeviceRouteBroker(Broker):
    controller = "device_routes"

    def show(self, **kwargs):
        """Shows the details for the specified device route.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param DeviceRouteID: The internal NetMRI identifier for this routing table entry on this device.
             :type DeviceRouteID: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param methods: A list of device route methods. The listed methods will be called on each device route returned and included in the output. Available methods are: data_source, device, interface, infradevice.
             :type methods: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param include: A list of associated object types to include in the output. The listed associations will be returned as outputs named according to the association name (see outputs below). Available includes are: data_source, device, interface.
             :type include: Array of String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return device_route: The device route identified by the specified DeviceRouteID.
             :rtype device_route: DeviceRoute

            """

        return self.api_request(self._get_method_fullname("show"), kwargs)

    def index(self, **kwargs):
        """Lists the available device routes. Any of the inputs listed may be be used to narrow the list; other inputs will be ignored. Of the various ways to query lists, using this method is most efficient.

            **Inputs**

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceID: The internal NetMRI identifier for the device from which this routing table entry was collected.
             :type DeviceID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceID: The internal NetMRI identifier for the device from which this routing table entry was collected.
             :type DeviceID: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceRouteID: The internal NetMRI identifier for this routing table entry on this device.
             :type DeviceRouteID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceRouteID: The internal NetMRI identifier for this routing table entry on this device.
             :type DeviceRouteID: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param InterfaceID: The internal NetMRI identifier of the outgoing interface for this route.
             :type InterfaceID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param InterfaceID: The internal NetMRI identifier of the outgoing interface for this route.
             :type InterfaceID: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteCIDR: The route destination network in CIDR format.
             :type RouteCIDR: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteCIDR: The route destination network in CIDR format.
             :type RouteCIDR: Array of String

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteEndTime: The ending effective time of this revision of this record, or empty if still in effect.
             :type RouteEndTime: DateTime

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteEndTime: The ending effective time of this revision of this record, or empty if still in effect.
             :type RouteEndTime: Array of DateTime

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteSubnetIPNumeric: The numerical value of the route destination network address.
             :type RouteSubnetIPNumeric: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteSubnetIPNumeric: The numerical value of the route destination network address.
             :type RouteSubnetIPNumeric: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param VirtualNetworkMemberID: The internal NetMRI identifier for the VRF-based VPN related to this record.
             :type VirtualNetworkMemberID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param VirtualNetworkMemberID: The internal NetMRI identifier for the VRF-based VPN related to this record.
             :type VirtualNetworkMemberID: Array of Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceGroupID: The internal NetMRI identifier of the device groups to which to limit the results.
             :type DeviceGroupID: Array of Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param timestamp: The data returned will represent the device routes as of this date and time. If omitted, the result will indicate the most recently collected data.
             :type timestamp: DateTime

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param methods: A list of device route methods. The listed methods will be called on each device route returned and included in the output. Available methods are: data_source, device, interface, infradevice.
             :type methods: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param include: A list of associated object types to include in the output. The listed associations will be returned as outputs named according to the association name (see outputs below). Available includes are: data_source, device, interface.
             :type include: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 0

             :param start: The record number to return in the selected page of data. It will always appear, although it may not be the first record. See the :limit for more information.
             :type start: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 1000

             :param limit: The size of the page of data, that is, the maximum number of records returned. The limit size will be used to break the data up into pages and the first page with the start record will be returned. So if you have 100 records and use a :limit of 10 and a :start of 10, you will get records 10-19. The maximum limit is 10000.
             :type limit: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` DeviceRouteID

             :param sort: The data field(s) to use for sorting the output. Default is DeviceRouteID. Valid values are DeviceRouteID, RouteStartTime, RouteEndTime, RouteChangedCols, RouteTimestamp, DataSourceID, DeviceID, InterfaceID, RouteSubnetIPDotted, RouteSubnetIPNumeric, RouteCIDR, RouteIfIndex, RouteNetMaskDotted, RouteNetMaskNumeric, RouteMetric1, RouteMetric2, RouteNextHopIPDotted, RouteNextHopIPNumeric, RouteProto, RouteType, RouteAdminDistance, VirtualNetworkMemberID.
             :type sort: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` asc

             :param dir: The direction(s) in which to sort the data. Default is 'asc'. Valid values are 'asc' and 'desc'.
             :type dir: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param select: The list of attributes to return for each DeviceRoute. Valid values are DeviceRouteID, RouteStartTime, RouteEndTime, RouteChangedCols, RouteTimestamp, DataSourceID, DeviceID, InterfaceID, RouteSubnetIPDotted, RouteSubnetIPNumeric, RouteCIDR, RouteIfIndex, RouteNetMaskDotted, RouteNetMaskNumeric, RouteMetric1, RouteMetric2, RouteNextHopIPDotted, RouteNextHopIPNumeric, RouteProto, RouteType, RouteAdminDistance, VirtualNetworkMemberID. If empty or omitted, all attributes will be returned.
             :type select: Array

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_field: The field name for NIOS GOTO that is used for locating a row position of records.
             :type goto_field: String

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_value: The value of goto_field for NIOS GOTO that is used for locating a row position of records.
             :type goto_value: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return device_routes: An array of the DeviceRoute objects that match the specified input criteria.
             :rtype device_routes: Array of DeviceRoute

            """

        return self.api_list_request(self._get_method_fullname("index"), kwargs)

    def search(self, **kwargs):
        """Lists the available device routes matching the input criteria. This method provides a more flexible search interface than the index method, but searching using this method is more demanding on the system and will not perform to the same level as the index method. The input fields listed below will be used as in the index method, to filter the result, along with the optional query string and XML filter described below.

            **Inputs**

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DataSourceID: The internal NetMRI identifier for the collector NetMRI that collected this data record.
             :type DataSourceID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DataSourceID: The internal NetMRI identifier for the collector NetMRI that collected this data record.
             :type DataSourceID: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceID: The internal NetMRI identifier for the device from which this routing table entry was collected.
             :type DeviceID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceID: The internal NetMRI identifier for the device from which this routing table entry was collected.
             :type DeviceID: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceRouteID: The internal NetMRI identifier for this routing table entry on this device.
             :type DeviceRouteID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceRouteID: The internal NetMRI identifier for this routing table entry on this device.
             :type DeviceRouteID: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param InterfaceID: The internal NetMRI identifier of the outgoing interface for this route.
             :type InterfaceID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param InterfaceID: The internal NetMRI identifier of the outgoing interface for this route.
             :type InterfaceID: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteAdminDistance: The administrative distance of the protocol through which this route was learned, as specified by default Cisco conventions.
             :type RouteAdminDistance: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteAdminDistance: The administrative distance of the protocol through which this route was learned, as specified by default Cisco conventions.
             :type RouteAdminDistance: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteCIDR: The route destination network in CIDR format.
             :type RouteCIDR: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteCIDR: The route destination network in CIDR format.
             :type RouteCIDR: Array of String

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteChangedCols: The fields that changed between this revision of the record and the previous revision.
             :type RouteChangedCols: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteChangedCols: The fields that changed between this revision of the record and the previous revision.
             :type RouteChangedCols: Array of String

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteEndTime: The ending effective time of this revision of this record, or empty if still in effect.
             :type RouteEndTime: DateTime

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteEndTime: The ending effective time of this revision of this record, or empty if still in effect.
             :type RouteEndTime: Array of DateTime

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteIfIndex: The SNMP interface index of the outgoing interface for this route.
             :type RouteIfIndex: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteIfIndex: The SNMP interface index of the outgoing interface for this route.
             :type RouteIfIndex: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteMetric1: The first route metric value.
             :type RouteMetric1: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteMetric1: The first route metric value.
             :type RouteMetric1: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteMetric2: The second route metric value.
             :type RouteMetric2: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteMetric2: The second route metric value.
             :type RouteMetric2: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteNetMaskDotted: The network mask of the route destination network in dotted (or colon-delimited for IPv6) format.
             :type RouteNetMaskDotted: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteNetMaskDotted: The network mask of the route destination network in dotted (or colon-delimited for IPv6) format.
             :type RouteNetMaskDotted: Array of String

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteNetMaskNumeric: The numerical value of the network mask.
             :type RouteNetMaskNumeric: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteNetMaskNumeric: The numerical value of the network mask.
             :type RouteNetMaskNumeric: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteNextHopIPDotted: The next hop IP address for this route, in dotted (or colon-delimited for IPv6) format.
             :type RouteNextHopIPDotted: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteNextHopIPDotted: The next hop IP address for this route, in dotted (or colon-delimited for IPv6) format.
             :type RouteNextHopIPDotted: Array of String

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteNextHopIPNumeric: The numerical value of the next hop IP address.
             :type RouteNextHopIPNumeric: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteNextHopIPNumeric: The numerical value of the next hop IP address.
             :type RouteNextHopIPNumeric: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteProto: The routing protocol through which this route was learned.
             :type RouteProto: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteProto: The routing protocol through which this route was learned.
             :type RouteProto: Array of String

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteStartTime: The starting effective time of this revision of the record.
             :type RouteStartTime: DateTime

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteStartTime: The starting effective time of this revision of the record.
             :type RouteStartTime: Array of DateTime

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteSubnetIPDotted: The route destination network address in dotted (or colon-delimited for IPv6) format.
             :type RouteSubnetIPDotted: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteSubnetIPDotted: The route destination network address in dotted (or colon-delimited for IPv6) format.
             :type RouteSubnetIPDotted: Array of String

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteSubnetIPNumeric: The numerical value of the route destination network address.
             :type RouteSubnetIPNumeric: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteSubnetIPNumeric: The numerical value of the route destination network address.
             :type RouteSubnetIPNumeric: Array of Integer

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteTimestamp: The date and time this record was collected or calculated.
             :type RouteTimestamp: DateTime

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteTimestamp: The date and time this record was collected or calculated.
             :type RouteTimestamp: Array of DateTime

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param RouteType: The type of the route.
             :type RouteType: String

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param RouteType: The type of the route.
             :type RouteType: Array of String

            |  ``api version min:`` 2.3
            |  ``api version max:`` 2.4
            |  ``required:`` False
            |  ``default:`` None

             :param VirtualNetworkMemberID: The internal NetMRI identifier for the VRF-based VPN related to this record.
             :type VirtualNetworkMemberID: Integer

            |  ``api version min:`` 2.5
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param VirtualNetworkMemberID: The internal NetMRI identifier for the VRF-based VPN related to this record.
             :type VirtualNetworkMemberID: Array of Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceGroupID: The internal NetMRI identifier of the device groups to which to limit the results.
             :type DeviceGroupID: Array of Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param timestamp: The data returned will represent the device routes as of this date and time. If omitted, the result will indicate the most recently collected data.
             :type timestamp: DateTime

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param methods: A list of device route methods. The listed methods will be called on each device route returned and included in the output. Available methods are: data_source, device, interface, infradevice.
             :type methods: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param include: A list of associated object types to include in the output. The listed associations will be returned as outputs named according to the association name (see outputs below). Available includes are: data_source, device, interface.
             :type include: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 0

             :param start: The record number to return in the selected page of data. It will always appear, although it may not be the first record. See the :limit for more information.
             :type start: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 1000

             :param limit: The size of the page of data, that is, the maximum number of records returned. The limit size will be used to break the data up into pages and the first page with the start record will be returned. So if you have 100 records and use a :limit of 10 and a :start of 10, you will get records 10-19. The maximum limit is 10000.
             :type limit: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` DeviceRouteID

             :param sort: The data field(s) to use for sorting the output. Default is DeviceRouteID. Valid values are DeviceRouteID, RouteStartTime, RouteEndTime, RouteChangedCols, RouteTimestamp, DataSourceID, DeviceID, InterfaceID, RouteSubnetIPDotted, RouteSubnetIPNumeric, RouteCIDR, RouteIfIndex, RouteNetMaskDotted, RouteNetMaskNumeric, RouteMetric1, RouteMetric2, RouteNextHopIPDotted, RouteNextHopIPNumeric, RouteProto, RouteType, RouteAdminDistance, VirtualNetworkMemberID.
             :type sort: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` asc

             :param dir: The direction(s) in which to sort the data. Default is 'asc'. Valid values are 'asc' and 'desc'.
             :type dir: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param select: The list of attributes to return for each DeviceRoute. Valid values are DeviceRouteID, RouteStartTime, RouteEndTime, RouteChangedCols, RouteTimestamp, DataSourceID, DeviceID, InterfaceID, RouteSubnetIPDotted, RouteSubnetIPNumeric, RouteCIDR, RouteIfIndex, RouteNetMaskDotted, RouteNetMaskNumeric, RouteMetric1, RouteMetric2, RouteNextHopIPDotted, RouteNextHopIPNumeric, RouteProto, RouteType, RouteAdminDistance, VirtualNetworkMemberID. If empty or omitted, all attributes will be returned.
             :type select: Array

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_field: The field name for NIOS GOTO that is used for locating a row position of records.
             :type goto_field: String

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_value: The value of goto_field for NIOS GOTO that is used for locating a row position of records.
             :type goto_value: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param query: This value will be matched against device routes, looking to see if one or more of the listed attributes contain the passed value. You may also surround the value with '/' and '/' to perform a regular expression search rather than a containment operation. Any record that matches will be returned. The attributes searched are: DataSourceID, DeviceID, DeviceRouteID, InterfaceID, RouteAdminDistance, RouteCIDR, RouteChangedCols, RouteEndTime, RouteIfIndex, RouteMetric1, RouteMetric2, RouteNetMaskDotted, RouteNetMaskNumeric, RouteNextHopIPDotted, RouteNextHopIPNumeric, RouteProto, RouteStartTime, RouteSubnetIPDotted, RouteSubnetIPNumeric, RouteTimestamp, RouteType, VirtualNetworkMemberID.
             :type query: String

            |  ``api version min:`` 2.3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param xml_filter: A SetFilter XML structure to further refine the search. The SetFilter will be applied AFTER any search query or field values, but before any limit options. The limit and pagination will be enforced after the filter. Remind that this kind of filter may be costly and inefficient if not associated with a database filtering.
             :type xml_filter: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return device_routes: An array of the DeviceRoute objects that match the specified input criteria.
             :rtype device_routes: Array of DeviceRoute

            """

        return self.api_list_request(self._get_method_fullname("search"), kwargs)

    def find(self, **kwargs):
        """Lists the available device routes matching the input specification. This provides the most flexible search specification of all the query mechanisms, enabling searching using comparison operations other than equality. However, it is more complex to use and will not perform as efficiently as the index or search methods. In the input descriptions below, 'field names' refers to the following fields: DataSourceID, DeviceID, DeviceRouteID, InterfaceID, RouteAdminDistance, RouteCIDR, RouteChangedCols, RouteEndTime, RouteIfIndex, RouteMetric1, RouteMetric2, RouteNetMaskDotted, RouteNetMaskNumeric, RouteNextHopIPDotted, RouteNextHopIPNumeric, RouteProto, RouteStartTime, RouteSubnetIPDotted, RouteSubnetIPNumeric, RouteTimestamp, RouteType, VirtualNetworkMemberID.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_DataSourceID: The operator to apply to the field DataSourceID. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. DataSourceID: The internal NetMRI identifier for the collector NetMRI that collected this data record. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_DataSourceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_DataSourceID: If op_DataSourceID is specified, the field named in this input will be compared to the value in DataSourceID using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_DataSourceID must be specified if op_DataSourceID is specified.
             :type val_f_DataSourceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_DataSourceID: If op_DataSourceID is specified, this value will be compared to the value in DataSourceID using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_DataSourceID must be specified if op_DataSourceID is specified.
             :type val_c_DataSourceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_DeviceID: The operator to apply to the field DeviceID. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. DeviceID: The internal NetMRI identifier for the device from which this routing table entry was collected. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_DeviceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_DeviceID: If op_DeviceID is specified, the field named in this input will be compared to the value in DeviceID using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_DeviceID must be specified if op_DeviceID is specified.
             :type val_f_DeviceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_DeviceID: If op_DeviceID is specified, this value will be compared to the value in DeviceID using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_DeviceID must be specified if op_DeviceID is specified.
             :type val_c_DeviceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_DeviceRouteID: The operator to apply to the field DeviceRouteID. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. DeviceRouteID: The internal NetMRI identifier for this routing table entry on this device. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_DeviceRouteID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_DeviceRouteID: If op_DeviceRouteID is specified, the field named in this input will be compared to the value in DeviceRouteID using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_DeviceRouteID must be specified if op_DeviceRouteID is specified.
             :type val_f_DeviceRouteID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_DeviceRouteID: If op_DeviceRouteID is specified, this value will be compared to the value in DeviceRouteID using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_DeviceRouteID must be specified if op_DeviceRouteID is specified.
             :type val_c_DeviceRouteID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_InterfaceID: The operator to apply to the field InterfaceID. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. InterfaceID: The internal NetMRI identifier of the outgoing interface for this route. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_InterfaceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_InterfaceID: If op_InterfaceID is specified, the field named in this input will be compared to the value in InterfaceID using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_InterfaceID must be specified if op_InterfaceID is specified.
             :type val_f_InterfaceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_InterfaceID: If op_InterfaceID is specified, this value will be compared to the value in InterfaceID using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_InterfaceID must be specified if op_InterfaceID is specified.
             :type val_c_InterfaceID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteAdminDistance: The operator to apply to the field RouteAdminDistance. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteAdminDistance: The administrative distance of the protocol through which this route was learned, as specified by default Cisco conventions. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteAdminDistance: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteAdminDistance: If op_RouteAdminDistance is specified, the field named in this input will be compared to the value in RouteAdminDistance using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteAdminDistance must be specified if op_RouteAdminDistance is specified.
             :type val_f_RouteAdminDistance: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteAdminDistance: If op_RouteAdminDistance is specified, this value will be compared to the value in RouteAdminDistance using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteAdminDistance must be specified if op_RouteAdminDistance is specified.
             :type val_c_RouteAdminDistance: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteCIDR: The operator to apply to the field RouteCIDR. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteCIDR: The route destination network in CIDR format. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteCIDR: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteCIDR: If op_RouteCIDR is specified, the field named in this input will be compared to the value in RouteCIDR using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteCIDR must be specified if op_RouteCIDR is specified.
             :type val_f_RouteCIDR: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteCIDR: If op_RouteCIDR is specified, this value will be compared to the value in RouteCIDR using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteCIDR must be specified if op_RouteCIDR is specified.
             :type val_c_RouteCIDR: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteChangedCols: The operator to apply to the field RouteChangedCols. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteChangedCols: The fields that changed between this revision of the record and the previous revision. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteChangedCols: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteChangedCols: If op_RouteChangedCols is specified, the field named in this input will be compared to the value in RouteChangedCols using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteChangedCols must be specified if op_RouteChangedCols is specified.
             :type val_f_RouteChangedCols: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteChangedCols: If op_RouteChangedCols is specified, this value will be compared to the value in RouteChangedCols using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteChangedCols must be specified if op_RouteChangedCols is specified.
             :type val_c_RouteChangedCols: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteEndTime: The operator to apply to the field RouteEndTime. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteEndTime: The ending effective time of this revision of this record, or empty if still in effect. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteEndTime: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteEndTime: If op_RouteEndTime is specified, the field named in this input will be compared to the value in RouteEndTime using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteEndTime must be specified if op_RouteEndTime is specified.
             :type val_f_RouteEndTime: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteEndTime: If op_RouteEndTime is specified, this value will be compared to the value in RouteEndTime using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteEndTime must be specified if op_RouteEndTime is specified.
             :type val_c_RouteEndTime: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteIfIndex: The operator to apply to the field RouteIfIndex. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteIfIndex: The SNMP interface index of the outgoing interface for this route. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteIfIndex: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteIfIndex: If op_RouteIfIndex is specified, the field named in this input will be compared to the value in RouteIfIndex using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteIfIndex must be specified if op_RouteIfIndex is specified.
             :type val_f_RouteIfIndex: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteIfIndex: If op_RouteIfIndex is specified, this value will be compared to the value in RouteIfIndex using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteIfIndex must be specified if op_RouteIfIndex is specified.
             :type val_c_RouteIfIndex: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteMetric1: The operator to apply to the field RouteMetric1. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteMetric1: The first route metric value. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteMetric1: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteMetric1: If op_RouteMetric1 is specified, the field named in this input will be compared to the value in RouteMetric1 using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteMetric1 must be specified if op_RouteMetric1 is specified.
             :type val_f_RouteMetric1: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteMetric1: If op_RouteMetric1 is specified, this value will be compared to the value in RouteMetric1 using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteMetric1 must be specified if op_RouteMetric1 is specified.
             :type val_c_RouteMetric1: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteMetric2: The operator to apply to the field RouteMetric2. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteMetric2: The second route metric value. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteMetric2: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteMetric2: If op_RouteMetric2 is specified, the field named in this input will be compared to the value in RouteMetric2 using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteMetric2 must be specified if op_RouteMetric2 is specified.
             :type val_f_RouteMetric2: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteMetric2: If op_RouteMetric2 is specified, this value will be compared to the value in RouteMetric2 using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteMetric2 must be specified if op_RouteMetric2 is specified.
             :type val_c_RouteMetric2: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteNetMaskDotted: The operator to apply to the field RouteNetMaskDotted. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteNetMaskDotted: The network mask of the route destination network in dotted (or colon-delimited for IPv6) format. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteNetMaskDotted: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteNetMaskDotted: If op_RouteNetMaskDotted is specified, the field named in this input will be compared to the value in RouteNetMaskDotted using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteNetMaskDotted must be specified if op_RouteNetMaskDotted is specified.
             :type val_f_RouteNetMaskDotted: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteNetMaskDotted: If op_RouteNetMaskDotted is specified, this value will be compared to the value in RouteNetMaskDotted using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteNetMaskDotted must be specified if op_RouteNetMaskDotted is specified.
             :type val_c_RouteNetMaskDotted: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteNetMaskNumeric: The operator to apply to the field RouteNetMaskNumeric. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteNetMaskNumeric: The numerical value of the network mask. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteNetMaskNumeric: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteNetMaskNumeric: If op_RouteNetMaskNumeric is specified, the field named in this input will be compared to the value in RouteNetMaskNumeric using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteNetMaskNumeric must be specified if op_RouteNetMaskNumeric is specified.
             :type val_f_RouteNetMaskNumeric: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteNetMaskNumeric: If op_RouteNetMaskNumeric is specified, this value will be compared to the value in RouteNetMaskNumeric using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteNetMaskNumeric must be specified if op_RouteNetMaskNumeric is specified.
             :type val_c_RouteNetMaskNumeric: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteNextHopIPDotted: The operator to apply to the field RouteNextHopIPDotted. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteNextHopIPDotted: The next hop IP address for this route, in dotted (or colon-delimited for IPv6) format. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteNextHopIPDotted: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteNextHopIPDotted: If op_RouteNextHopIPDotted is specified, the field named in this input will be compared to the value in RouteNextHopIPDotted using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteNextHopIPDotted must be specified if op_RouteNextHopIPDotted is specified.
             :type val_f_RouteNextHopIPDotted: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteNextHopIPDotted: If op_RouteNextHopIPDotted is specified, this value will be compared to the value in RouteNextHopIPDotted using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteNextHopIPDotted must be specified if op_RouteNextHopIPDotted is specified.
             :type val_c_RouteNextHopIPDotted: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteNextHopIPNumeric: The operator to apply to the field RouteNextHopIPNumeric. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteNextHopIPNumeric: The numerical value of the next hop IP address. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteNextHopIPNumeric: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteNextHopIPNumeric: If op_RouteNextHopIPNumeric is specified, the field named in this input will be compared to the value in RouteNextHopIPNumeric using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteNextHopIPNumeric must be specified if op_RouteNextHopIPNumeric is specified.
             :type val_f_RouteNextHopIPNumeric: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteNextHopIPNumeric: If op_RouteNextHopIPNumeric is specified, this value will be compared to the value in RouteNextHopIPNumeric using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteNextHopIPNumeric must be specified if op_RouteNextHopIPNumeric is specified.
             :type val_c_RouteNextHopIPNumeric: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteProto: The operator to apply to the field RouteProto. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteProto: The routing protocol through which this route was learned. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteProto: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteProto: If op_RouteProto is specified, the field named in this input will be compared to the value in RouteProto using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteProto must be specified if op_RouteProto is specified.
             :type val_f_RouteProto: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteProto: If op_RouteProto is specified, this value will be compared to the value in RouteProto using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteProto must be specified if op_RouteProto is specified.
             :type val_c_RouteProto: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteStartTime: The operator to apply to the field RouteStartTime. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteStartTime: The starting effective time of this revision of the record. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteStartTime: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteStartTime: If op_RouteStartTime is specified, the field named in this input will be compared to the value in RouteStartTime using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteStartTime must be specified if op_RouteStartTime is specified.
             :type val_f_RouteStartTime: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteStartTime: If op_RouteStartTime is specified, this value will be compared to the value in RouteStartTime using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteStartTime must be specified if op_RouteStartTime is specified.
             :type val_c_RouteStartTime: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteSubnetIPDotted: The operator to apply to the field RouteSubnetIPDotted. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteSubnetIPDotted: The route destination network address in dotted (or colon-delimited for IPv6) format. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteSubnetIPDotted: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteSubnetIPDotted: If op_RouteSubnetIPDotted is specified, the field named in this input will be compared to the value in RouteSubnetIPDotted using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteSubnetIPDotted must be specified if op_RouteSubnetIPDotted is specified.
             :type val_f_RouteSubnetIPDotted: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteSubnetIPDotted: If op_RouteSubnetIPDotted is specified, this value will be compared to the value in RouteSubnetIPDotted using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteSubnetIPDotted must be specified if op_RouteSubnetIPDotted is specified.
             :type val_c_RouteSubnetIPDotted: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteSubnetIPNumeric: The operator to apply to the field RouteSubnetIPNumeric. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteSubnetIPNumeric: The numerical value of the route destination network address. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteSubnetIPNumeric: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteSubnetIPNumeric: If op_RouteSubnetIPNumeric is specified, the field named in this input will be compared to the value in RouteSubnetIPNumeric using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteSubnetIPNumeric must be specified if op_RouteSubnetIPNumeric is specified.
             :type val_f_RouteSubnetIPNumeric: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteSubnetIPNumeric: If op_RouteSubnetIPNumeric is specified, this value will be compared to the value in RouteSubnetIPNumeric using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteSubnetIPNumeric must be specified if op_RouteSubnetIPNumeric is specified.
             :type val_c_RouteSubnetIPNumeric: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteTimestamp: The operator to apply to the field RouteTimestamp. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteTimestamp: The date and time this record was collected or calculated. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteTimestamp: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteTimestamp: If op_RouteTimestamp is specified, the field named in this input will be compared to the value in RouteTimestamp using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteTimestamp must be specified if op_RouteTimestamp is specified.
             :type val_f_RouteTimestamp: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteTimestamp: If op_RouteTimestamp is specified, this value will be compared to the value in RouteTimestamp using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteTimestamp must be specified if op_RouteTimestamp is specified.
             :type val_c_RouteTimestamp: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_RouteType: The operator to apply to the field RouteType. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. RouteType: The type of the route. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_RouteType: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_RouteType: If op_RouteType is specified, the field named in this input will be compared to the value in RouteType using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_RouteType must be specified if op_RouteType is specified.
             :type val_f_RouteType: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_RouteType: If op_RouteType is specified, this value will be compared to the value in RouteType using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_RouteType must be specified if op_RouteType is specified.
             :type val_c_RouteType: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_VirtualNetworkMemberID: The operator to apply to the field VirtualNetworkMemberID. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. VirtualNetworkMemberID: The internal NetMRI identifier for the VRF-based VPN related to this record. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_VirtualNetworkMemberID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_VirtualNetworkMemberID: If op_VirtualNetworkMemberID is specified, the field named in this input will be compared to the value in VirtualNetworkMemberID using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_VirtualNetworkMemberID must be specified if op_VirtualNetworkMemberID is specified.
             :type val_f_VirtualNetworkMemberID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_VirtualNetworkMemberID: If op_VirtualNetworkMemberID is specified, this value will be compared to the value in VirtualNetworkMemberID using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_VirtualNetworkMemberID must be specified if op_VirtualNetworkMemberID is specified.
             :type val_c_VirtualNetworkMemberID: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param DeviceGroupID: The internal NetMRI identifier of the device groups to which to limit the results.
             :type DeviceGroupID: Array of Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param timestamp: The data returned will represent the device routes as of this date and time. If omitted, the result will indicate the most recently collected data.
             :type timestamp: DateTime

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param methods: A list of device route methods. The listed methods will be called on each device route returned and included in the output. Available methods are: data_source, device, interface, infradevice.
             :type methods: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param include: A list of associated object types to include in the output. The listed associations will be returned as outputs named according to the association name (see outputs below). Available includes are: data_source, device, interface.
             :type include: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 0

             :param start: The record number to return in the selected page of data. It will always appear, although it may not be the first record. See the :limit for more information.
             :type start: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 1000

             :param limit: The size of the page of data, that is, the maximum number of records returned. The limit size will be used to break the data up into pages and the first page with the start record will be returned. So if you have 100 records and use a :limit of 10 and a :start of 10, you will get records 10-19. The maximum limit is 10000.
             :type limit: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` DeviceRouteID

             :param sort: The data field(s) to use for sorting the output. Default is DeviceRouteID. Valid values are DeviceRouteID, RouteStartTime, RouteEndTime, RouteChangedCols, RouteTimestamp, DataSourceID, DeviceID, InterfaceID, RouteSubnetIPDotted, RouteSubnetIPNumeric, RouteCIDR, RouteIfIndex, RouteNetMaskDotted, RouteNetMaskNumeric, RouteMetric1, RouteMetric2, RouteNextHopIPDotted, RouteNextHopIPNumeric, RouteProto, RouteType, RouteAdminDistance, VirtualNetworkMemberID.
             :type sort: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` asc

             :param dir: The direction(s) in which to sort the data. Default is 'asc'. Valid values are 'asc' and 'desc'.
             :type dir: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param select: The list of attributes to return for each DeviceRoute. Valid values are DeviceRouteID, RouteStartTime, RouteEndTime, RouteChangedCols, RouteTimestamp, DataSourceID, DeviceID, InterfaceID, RouteSubnetIPDotted, RouteSubnetIPNumeric, RouteCIDR, RouteIfIndex, RouteNetMaskDotted, RouteNetMaskNumeric, RouteMetric1, RouteMetric2, RouteNextHopIPDotted, RouteNextHopIPNumeric, RouteProto, RouteType, RouteAdminDistance, VirtualNetworkMemberID. If empty or omitted, all attributes will be returned.
             :type select: Array

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_field: The field name for NIOS GOTO that is used for locating a row position of records.
             :type goto_field: String

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_value: The value of goto_field for NIOS GOTO that is used for locating a row position of records.
             :type goto_value: String

            |  ``api version min:`` 2.3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param xml_filter: A SetFilter XML structure to further refine the search. The SetFilter will be applied AFTER any search query or field values, but before any limit options. The limit and pagination will be enforced after the filter. Remind that this kind of filter may be costly and inefficient if not associated with a database filtering.
             :type xml_filter: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return device_routes: An array of the DeviceRoute objects that match the specified input criteria.
             :rtype device_routes: Array of DeviceRoute

            """

        return self.api_list_request(self._get_method_fullname("find"), kwargs)

    def data_source(self, **kwargs):
        """The NetMRI device that collected this record.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param DeviceRouteID: The internal NetMRI identifier for this routing table entry on this device.
             :type DeviceRouteID: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return : The NetMRI device that collected this record.
             :rtype : DataSource

            """

        return self.api_request(self._get_method_fullname("data_source"), kwargs)

    def interface(self, **kwargs):
        """The outgoing interface for this route.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param DeviceRouteID: The internal NetMRI identifier for this routing table entry on this device.
             :type DeviceRouteID: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return : The outgoing interface for this route.
             :rtype : Interface

            """

        return self.api_request(self._get_method_fullname("interface"), kwargs)

    def infradevice(self, **kwargs):
        """The device from which this routing table entry was collected.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param DeviceRouteID: The internal NetMRI identifier for this routing table entry on this device.
             :type DeviceRouteID: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return : The device from which this routing table entry was collected.
             :rtype : InfraDevice

            """

        return self.api_request(self._get_method_fullname("infradevice"), kwargs)

    def device(self, **kwargs):
        """The device from which this routing table entry was collected.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param DeviceRouteID: The internal NetMRI identifier for this routing table entry on this device.
             :type DeviceRouteID: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return : The device from which this routing table entry was collected.
             :rtype : Device

            """

        return self.api_request(self._get_method_fullname("device"), kwargs)



from ..broker import Broker


class GuiStateBroker(Broker):
    controller = "gui_states"

    def show(self, **kwargs):
        """Shows the details for the specified gui state.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: The internal NetMRI identifier of the gui state.
             :type id: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return gui_state: The gui state identified by the specified id.
             :rtype gui_state: GuiState

            """

        return self.api_request(self._get_method_fullname("show"), kwargs)

    def index(self, **kwargs):
        """Lists the available gui states. Any of the inputs listed may be be used to narrow the list; other inputs will be ignored. Of the various ways to query lists, using this method is most efficient.

            **Inputs**

            |  ``api version min:`` 3.3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param id: No description is available for id.
             :type id: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 0

             :param start: The record number to return in the selected page of data. It will always appear, although it may not be the first record. See the :limit for more information.
             :type start: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 1000

             :param limit: The size of the page of data, that is, the maximum number of records returned. The limit size will be used to break the data up into pages and the first page with the start record will be returned. So if you have 100 records and use a :limit of 10 and a :start of 10, you will get records 10-19. The maximum limit is 10000.
             :type limit: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` id

             :param sort: The data field(s) to use for sorting the output. Default is id. Valid values are id, object_name, view_name, auth_user_id, object_state, created_at, updated_at, shared, desc, default.
             :type sort: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` asc

             :param dir: The direction(s) in which to sort the data. Default is 'asc'. Valid values are 'asc' and 'desc'.
             :type dir: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param select: The list of attributes to return for each GuiState. Valid values are id, object_name, view_name, auth_user_id, object_state, created_at, updated_at, shared, desc, default. If empty or omitted, all attributes will be returned.
             :type select: Array

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_field: The field name for NIOS GOTO that is used for locating a row position of records.
             :type goto_field: String

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_value: The value of goto_field for NIOS GOTO that is used for locating a row position of records.
             :type goto_value: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return gui_states: An array of the GuiState objects that match the specified input criteria.
             :rtype gui_states: Array of GuiState

            """

        return self.api_list_request(self._get_method_fullname("index"), kwargs)

    def find(self, **kwargs):
        """Lists the available gui states matching the input specification. This provides the most flexible search specification of all the query mechanisms, enabling searching using comparison operations other than equality. However, it is more complex to use and will not perform as efficiently as the index or search methods. In the input descriptions below, 'field names' refers to the following fields: auth_user_id, created_at, default, desc, id, object_name, object_state, shared, updated_at, view_name.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_auth_user_id: The operator to apply to the field auth_user_id. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. auth_user_id: No description is available for auth_user_id. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_auth_user_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_auth_user_id: If op_auth_user_id is specified, the field named in this input will be compared to the value in auth_user_id using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_auth_user_id must be specified if op_auth_user_id is specified.
             :type val_f_auth_user_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_auth_user_id: If op_auth_user_id is specified, this value will be compared to the value in auth_user_id using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_auth_user_id must be specified if op_auth_user_id is specified.
             :type val_c_auth_user_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_created_at: The operator to apply to the field created_at. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. created_at: No description is available for created_at. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_created_at: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_created_at: If op_created_at is specified, the field named in this input will be compared to the value in created_at using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_created_at must be specified if op_created_at is specified.
             :type val_f_created_at: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_created_at: If op_created_at is specified, this value will be compared to the value in created_at using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_created_at must be specified if op_created_at is specified.
             :type val_c_created_at: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_default: The operator to apply to the field default. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. default: No description is available for default. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_default: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_default: If op_default is specified, the field named in this input will be compared to the value in default using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_default must be specified if op_default is specified.
             :type val_f_default: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_default: If op_default is specified, this value will be compared to the value in default using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_default must be specified if op_default is specified.
             :type val_c_default: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_desc: The operator to apply to the field desc. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. desc: No description is available for desc. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_desc: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_desc: If op_desc is specified, the field named in this input will be compared to the value in desc using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_desc must be specified if op_desc is specified.
             :type val_f_desc: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_desc: If op_desc is specified, this value will be compared to the value in desc using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_desc must be specified if op_desc is specified.
             :type val_c_desc: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_id: The operator to apply to the field id. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. id: No description is available for id. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_id: If op_id is specified, the field named in this input will be compared to the value in id using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_id must be specified if op_id is specified.
             :type val_f_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_id: If op_id is specified, this value will be compared to the value in id using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_id must be specified if op_id is specified.
             :type val_c_id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_object_name: The operator to apply to the field object_name. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. object_name: No description is available for object_name. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_object_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_object_name: If op_object_name is specified, the field named in this input will be compared to the value in object_name using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_object_name must be specified if op_object_name is specified.
             :type val_f_object_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_object_name: If op_object_name is specified, this value will be compared to the value in object_name using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_object_name must be specified if op_object_name is specified.
             :type val_c_object_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_object_state: The operator to apply to the field object_state. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. object_state: No description is available for object_state. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_object_state: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_object_state: If op_object_state is specified, the field named in this input will be compared to the value in object_state using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_object_state must be specified if op_object_state is specified.
             :type val_f_object_state: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_object_state: If op_object_state is specified, this value will be compared to the value in object_state using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_object_state must be specified if op_object_state is specified.
             :type val_c_object_state: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_shared: The operator to apply to the field shared. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. shared: No description is available for shared. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_shared: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_shared: If op_shared is specified, the field named in this input will be compared to the value in shared using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_shared must be specified if op_shared is specified.
             :type val_f_shared: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_shared: If op_shared is specified, this value will be compared to the value in shared using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_shared must be specified if op_shared is specified.
             :type val_c_shared: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_updated_at: The operator to apply to the field updated_at. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. updated_at: No description is available for updated_at. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_updated_at: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_updated_at: If op_updated_at is specified, the field named in this input will be compared to the value in updated_at using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_updated_at must be specified if op_updated_at is specified.
             :type val_f_updated_at: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_updated_at: If op_updated_at is specified, this value will be compared to the value in updated_at using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_updated_at must be specified if op_updated_at is specified.
             :type val_c_updated_at: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param op_view_name: The operator to apply to the field view_name. Valid values are: =, <>, rlike, not rlike, >, >=, <, <=, like, not like, is null, is not null, between. view_name: No description is available for view_name. For the between operator the value will be treated as an Array if comma delimited string is passed, and it must contain an even number of values.
             :type op_view_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_f_view_name: If op_view_name is specified, the field named in this input will be compared to the value in view_name using the specified operator. That is, the value in this input will be treated as another field name, rather than a constant value. Either this field or val_c_view_name must be specified if op_view_name is specified.
             :type val_f_view_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param val_c_view_name: If op_view_name is specified, this value will be compared to the value in view_name using the specified operator. The value in this input will be treated as an explicit constant value. Either this field or val_f_view_name must be specified if op_view_name is specified.
             :type val_c_view_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 0

             :param start: The record number to return in the selected page of data. It will always appear, although it may not be the first record. See the :limit for more information.
             :type start: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` 1000

             :param limit: The size of the page of data, that is, the maximum number of records returned. The limit size will be used to break the data up into pages and the first page with the start record will be returned. So if you have 100 records and use a :limit of 10 and a :start of 10, you will get records 10-19. The maximum limit is 10000.
             :type limit: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` id

             :param sort: The data field(s) to use for sorting the output. Default is id. Valid values are id, object_name, view_name, auth_user_id, object_state, created_at, updated_at, shared, desc, default.
             :type sort: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` asc

             :param dir: The direction(s) in which to sort the data. Default is 'asc'. Valid values are 'asc' and 'desc'.
             :type dir: Array of String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param select: The list of attributes to return for each GuiState. Valid values are id, object_name, view_name, auth_user_id, object_state, created_at, updated_at, shared, desc, default. If empty or omitted, all attributes will be returned.
             :type select: Array

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_field: The field name for NIOS GOTO that is used for locating a row position of records.
             :type goto_field: String

            |  ``api version min:`` 2.8
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param goto_value: The value of goto_field for NIOS GOTO that is used for locating a row position of records.
             :type goto_value: String

            |  ``api version min:`` 2.3
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param xml_filter: A SetFilter XML structure to further refine the search. The SetFilter will be applied AFTER any search query or field values, but before any limit options. The limit and pagination will be enforced after the filter. Remind that this kind of filter may be costly and inefficient if not associated with a database filtering.
             :type xml_filter: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return gui_states: An array of the GuiState objects that match the specified input criteria.
             :rtype gui_states: Array of GuiState

            """

        return self.api_list_request(self._get_method_fullname("find"), kwargs)

    def default(self, **kwargs):
        """Get default view

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param object_name: Internal identifier of a grid
             :type object_name: String

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("default"), kwargs)

    def create(self, **kwargs):
        """Create view

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param object_name: Internal identifier of a grid
             :type object_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param view_name: Name
             :type view_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param desc: Description
             :type desc: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param set_default: Set this view as default for the grid
             :type set_default: Boolean

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param shared: Make this view accessible to other users
             :type shared: Boolean

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param object_state: Configuration of the grid
             :type object_state: String

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("create"), kwargs)

    def update(self, **kwargs):
        """Update existing view

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: Internal identifier of grid view
             :type id: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param view_name: Name
             :type view_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param desc: Description
             :type desc: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param set_default: Set this view as default for the grid
             :type set_default: Boolean

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param shared: Make this view accessible to other users
             :type shared: Boolean

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param object_state: Configuration of the grid
             :type object_state: String

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("update"), kwargs)

    def delete(self, **kwargs):
        """Delete gui state

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: Internal identifier of gui state
             :type id: Integer

            **Outputs**

            """

        return self.api_request(self._get_method_fullname("delete"), kwargs)



from ..remote import RemoteModel


class DeviceRouteNetworkExplorerSummariesSummaryGridRemote(RemoteModel):
    """



    |  ``id:`` none
    |  ``attribute type:`` string

    |  ``DeviceID:`` none
    |  ``attribute type:`` string

    |  ``InterfaceID:`` none
    |  ``attribute type:`` string

    |  ``DeviceRouteID:`` none
    |  ``attribute type:`` string

    |  ``DeviceIPDotted:`` none
    |  ``attribute type:`` string

    |  ``DeviceIPNumeric:`` none
    |  ``attribute type:`` string

    |  ``VirtualNetworkID:`` none
    |  ``attribute type:`` string

    |  ``Network:`` none
    |  ``attribute type:`` string

    |  ``DeviceName:`` none
    |  ``attribute type:`` string

    |  ``ifDescr:`` none
    |  ``attribute type:`` string

    |  ``Interface:`` none
    |  ``attribute type:`` string

    |  ``VirtualNetworkMemberName:`` none
    |  ``attribute type:`` string

    |  ``VrfNetworkViewID:`` none
    |  ``attribute type:`` string

    |  ``VrfNetworkView:`` none
    |  ``attribute type:`` string

    |  ``InterfaceName:`` none
    |  ``attribute type:`` string

    |  ``InterfaceNameSort:`` none
    |  ``attribute type:`` string

    |  ``DeviceType:`` none
    |  ``attribute type:`` string

    |  ``RouteNextHopIPDotted:`` none
    |  ``attribute type:`` string

    |  ``RouteNextHopIPNumeric:`` none
    |  ``attribute type:`` string

    |  ``ifIndex:`` none
    |  ``attribute type:`` string

    |  ``RouteProto:`` none
    |  ``attribute type:`` string

    |  ``RouteCIDR:`` none
    |  ``attribute type:`` string

    |  ``RouteDistinguisher:`` none
    |  ``attribute type:`` string

    |  ``DeviceAssurance:`` none
    |  ``attribute type:`` string

    |  ``CIDRouteNumeric:`` none
    |  ``attribute type:`` string

    """

    properties = ("id",
                  "DeviceID",
                  "InterfaceID",
                  "DeviceRouteID",
                  "DeviceIPDotted",
                  "DeviceIPNumeric",
                  "VirtualNetworkID",
                  "Network",
                  "DeviceName",
                  "ifDescr",
                  "Interface",
                  "VirtualNetworkMemberName",
                  "VrfNetworkViewID",
                  "VrfNetworkView",
                  "InterfaceName",
                  "InterfaceNameSort",
                  "DeviceType",
                  "RouteNextHopIPDotted",
                  "RouteNextHopIPNumeric",
                  "ifIndex",
                  "RouteProto",
                  "RouteCIDR",
                  "RouteDistinguisher",
                  "DeviceAssurance",
                  "CIDRouteNumeric",
                  )



from ..broker import Broker


class CliConnectionBroker(Broker):
    controller = "cli_connections"

    def create(self, **kwargs):
        """Opens a CLI connection with the device.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: The DIS session id.
             :type id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param device_id: The NetMRI internal identifier for the device.
             :type device_id: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return cli_connection: The CLI connection object that represents a CLI connection with the device.
             :rtype cli_connection: CliConnection

            """

        return self.api_request(self._get_method_fullname("create"), kwargs)

    def open(self, **kwargs):
        """Opens a CLI connection with the device.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: The DIS session id.
             :type id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param device_id: The NetMRI internal identifier for the device.
             :type device_id: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return cli_connection: The CLI connection object that represents a CLI connection with the device.
             :rtype cli_connection: CliConnection

            """

        return self.api_request(self._get_method_fullname("open"), kwargs)

    def close(self, **kwargs):
        """Closes an open CLI connection with the device.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: The DIS session id.
             :type id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param device_id: The NetMRI internal identifier for the device.
             :type device_id: Integer

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return command_response: The response to sending the command.
             :rtype command_response: String

            """

        return self.api_request(self._get_method_fullname("close"), kwargs)

    def set_variable(self, **kwargs):
        """Set a local variable.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: The DIS session id.
             :type id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param device_id: The NetMRI internal identifier for the device.
             :type device_id: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param command: The command to set a local variable.
             :type command: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return command_response: The response to sending the command.
             :rtype command_response: String

            """

        return self.api_request(self._get_method_fullname("set_variable"), kwargs)

    def send_command(self, **kwargs):
        """Sends a command to the device.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: The DIS session id.
             :type id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param device_id: The NetMRI internal identifier for the device.
             :type device_id: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param command: The command to send to the device.
             :type command: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:``

             :param regex: Expected regex: prompt or question resulting of the command.
             :type regex: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` False

             :param debug: Option to test the command and not send to the device.
             :type debug: Boolean

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return command_response: The response to sending the command.
             :rtype command_response: String

            """

        return self.api_request(self._get_method_fullname("send_command"), kwargs)

    def get_template(self, **kwargs):
        """get template

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: The DIS session id.
             :type id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param device_id: The NetMRI internal identifier for the device.
             :type device_id: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param template: The name of the template that contains commands to be evaluated.
             :type template: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` False

             :param stage: Option to stage the template.
             :type stage: Boolean

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return command_response: The response to sending the command.
             :rtype command_response: String

            """

        return self.api_request(self._get_method_fullname("get_template"), kwargs)

    def get_list_value(self, **kwargs):
        """Set local variables from a list.

            **Inputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param id: The DIS session id.
             :type id: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param device_id: The NetMRI internal identifier for the device.
             :type device_id: Integer

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param list_name: Name or Short Name of the List
             :type list_name: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param key_column: Key column for the search
             :type key_column: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param key_value: Value which will be seached in the column
             :type key_value: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` True
            |  ``default:`` None

             :param value_column: Value of column which should be returned
             :type value_column: String

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :param default_value: Value which will be returned if nothing was found
             :type default_value: String

            **Outputs**

            |  ``api version min:`` None
            |  ``api version max:`` None
            |  ``required:`` False
            |  ``default:`` None

             :return command_response: The response to sending the command.
             :rtype command_response: String

            """

        return self.api_request(self._get_method_fullname("get_list_value"), kwargs)



from ..remote import RemoteModel
from infoblox_netmri.utils.utils import check_api_availability


class IfWirelessSSIDAuthRemote(RemoteModel):
    """
    This table list out the entries of WirelessSSID Authentication.


    |  ``IfWirelessSSIDAuthID:`` The internal NetMRI identifier of the Wireless SSID Authentication.
    |  ``attribute type:`` number

    |  ``DataSourceID:`` The internal NetMRI identifier for the collector NetMRI that collected this data record.
    |  ``attribute type:`` number


    |  ``DeviceID:`` The internal NetMRI identifier for the device from which wireless SSID information was collected.
    |  ``attribute type:`` number

    |  ``InterfaceID:`` The internal NetMRI identifier for the local interface for this Wireless SSID table entry.
    |  ``attribute type:`` number

    |  ``ifWirelessSSIDAuthStartTime:`` The starting effective time of this record.
    |  ``attribute type:`` datetime

    |  ``ifWirelessSSIDAuthEndTime:`` The ending effective time of this record, or empty if still in effect.
    |  ``attribute type:`` datetime

    |  ``ifWirelessSSIDAuthChangedCols:`` The fields that changed between this revision of the record and the previous revision.
    |  ``attribute type:`` string

    |  ``ifWirelessSSIDAuthTimestamp:`` The date and time this record was collected or calculated.
    |  ``attribute type:`` datetime

    |  ``SSIDIndex:`` The SSID index of the local interface for this Wireless SSID.
    |  ``attribute type:`` string

    |  ``SSIDAlgorithmIndex:`` The SSID algorithm index of the wireless SSID.
    |  ``attribute type:`` string

    |  ``SSIDAuthEnabledInd:`` A flag indicates SSID Authentication is enabled or not.
    |  ``attribute type:`` bool

    |  ``SSIDEAPRequiredInd:`` A flag indicates whether a extensible authentication protocol is required or not.
    |  ``attribute type:`` bool

    |  ``SSIDEAPMethod:`` The Extensible Authentication Protocol Method is used in theWirelessSSIDAuth.
    |  ``attribute type:`` string

    |  ``SSIDMACAuthRequiredInd:`` A flag indicates the SSID Media Access Controller(MAC) is required the authentication or not.
    |  ``attribute type:`` bool

    |  ``SSIDMACAuthMethod:`` The Media Access Controller(MAC) authentication method in the Wireless SSID.
    |  ``attribute type:`` string

    |  ``SSIDDefaultVlanIndex:`` The default VLAN index of the local interface of the Wireless SSID.
    |  ``attribute type:`` string

    |  ``VlanID:`` The internal NetMRI identifier of the VLAN.
    |  ``attribute type:`` number

    |  ``SSIDAuthAlgorithm:`` The SSID Authentication algorithm in the interface wireless.
    |  ``attribute type:`` string





    """

    properties = ("IfWirelessSSIDAuthID",
                  "DataSourceID",
                  "DeviceID",
                  "InterfaceID",
                  "ifWirelessSSIDAuthStartTime",
                  "ifWirelessSSIDAuthEndTime",
                  "ifWirelessSSIDAuthChangedCols",
                  "ifWirelessSSIDAuthTimestamp",
                  "SSIDIndex",
                  "SSIDAlgorithmIndex",
                  "SSIDAuthEnabledInd",
                  "SSIDEAPRequiredInd",
                  "SSIDEAPMethod",
                  "SSIDMACAuthRequiredInd",
                  "SSIDMACAuthMethod",
                  "SSIDDefaultVlanIndex",
                  "VlanID",
                  "SSIDAuthAlgorithm",
                  )

    @property
    @check_api_availability
    def data_source(self):
        """
        The collector NetMRI that collected this data record.
        ``attribute type:`` model
        """
        return self.broker.data_source(**{"IfWirelessSSIDAuthID": self.IfWirelessSSIDAuthID})

    @property
    @check_api_availability
    def device(self):
        """
        The device from which this data was collected.
        ``attribute type:`` model
        """
        return self.broker.device(**{"IfWirelessSSIDAuthID": self.IfWirelessSSIDAuthID})

    @property
    @check_api_availability
    def interface(self):
        """
        interface
        ``attribute type:`` model
        """
        return self.broker.interface(**{"IfWirelessSSIDAuthID": self.IfWirelessSSIDAuthID})

    @property
    @check_api_availability
    def vlan(self):
        """
        vlan
        ``attribute type:`` model
        """
        return self.broker.vlan(**{"IfWirelessSSIDAuthID": self.IfWirelessSSIDAuthID})

    @property
    @check_api_availability
    def infradevice(self):
        """
        The device from which this data was collected.
        ``attribute type:`` model
        """
        return self.broker.infradevice(**{"IfWirelessSSIDAuthID": self.IfWirelessSSIDAuthID})
